import { visitParents } from "unist-util-visit-parents";
import { fromHtml } from "hast-util-from-html";
import { renderEditor, renderCodeBlock } from "prism-code-editor/ssr";
const parseValue = (value) => {
  if (value && `"'{`.includes(value[0])) value = value.slice(1, -1);
  if (!value || value == "true") return true;
  if (value == "false") return false;
  if (value == "null") return null;
  if (!isNaN(+value)) return +value;
  return value;
};
const parseMeta = (meta) => {
  const result = {};
  const pattern = /([^\s"'{}=]+)(?:\s*=\s*("[^"]*"|'[^']*'|\{[^}]*\}|[^\s"'{}=]+))?/g;
  let match;
  while (match = pattern.exec(meta)) {
    let [, prop, value] = match;
    result[prop] = parseValue(value);
  }
  return result;
};
const createEditor = (props) => {
  const tokenizeCallback = props.tokenizeCallback;
  delete props.tokenizeCallback;
  return renderEditor(props, { tokenizeCallback });
};
const createCodeBlock = (props) => renderCodeBlock(props);
const rehypePrismCodeEditor = ({
  editorsOnly,
  defaultEditorProps,
  defaultCodeBlockProps,
  customRenderer
} = {}) => {
  return (tree) => {
    const nodes = [];
    visitParents(tree, { type: "element", tagName: "code" }, (node, ancestors) => {
      const [parent, pre] = ancestors.slice(-2);
      if (pre.tagName != "pre" || pre.children[1]) return;
      const children = node.children;
      const text = children[0];
      if (children[1] || text.type != "text") return;
      nodes.push([node, pre, parent, text.value.slice(0, -1)]);
    });
    nodes.forEach(([codeEl, pre, parent, code]) => {
      const properties = codeEl.properties;
      const meta = codeEl.data?.meta || properties.metastring || "";
      const props = parseMeta(meta);
      const isEditor = props.editor;
      if (editorsOnly && isEditor == null) return;
      props[isEditor ? "value" : "code"] = code;
      delete props.editor;
      let className = properties.className;
      if (Array.isArray(className)) className = className.join(" ");
      const lang = typeof className == "string" && /\blanguage-(\S+)/.exec(className)?.[1] || "text";
      props.language = lang.replace(/\b\+\+/g, "pp").replace(/\b#/g, "sharp");
      const renderFunc = isEditor ? createEditor : createCodeBlock;
      const merged = { ...isEditor ? defaultEditorProps : defaultCodeBlockProps, ...props };
      const html = customRenderer ? customRenderer(merged, renderFunc) : renderFunc(merged);
      const children = parent.children;
      children[children.indexOf(pre)] = fromHtml(html, { fragment: true }).children[0];
    });
  };
};
export {
  rehypePrismCodeEditor
};
//# sourceMappingURL=index.js.map
